/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package util.dao;

import entities.Reservation;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import util.DateConverter;

/**
 *
 * @author jelic
 */
public class ReservationDAO implements ReservationDAOInterface{

    @Override
    public boolean deleteReservation(Reservation r) {
        Connection conn = util.DB.getInstance().getConnection();
        try (PreparedStatement ps = conn.prepareStatement("delete from rezervacija where id=?");) {
            ps.setInt(1, r.getId());

            return ps.executeUpdate() != 0;

        } catch (SQLException ex) {
            Logger.getLogger(UserDAO.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            util.DB.getInstance().putConnection(conn);
        }

        return false;
    }

    @Override
    public List<Reservation> getAllReservations() {
        List<Reservation> all = new ArrayList<>();
        Connection conn = util.DB.getInstance().getConnection();
        try (PreparedStatement ps = conn.prepareStatement("select * from rezervacija order by datumOd");) {

            ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                Reservation r = new Reservation();
                r.setFrom(DateConverter.TimestampToLocalDateTime(rs.getTimestamp("datumOd")));
                r.setTo(DateConverter.TimestampToLocalDateTime(rs.getTimestamp("datumDo")));
                r.setTitle(rs.getString("naziv"));
                r.parseAndSetLabs(rs.getString("prostorije"));
                r.setId(rs.getInt("id"));
                all.add(r);
            }

        } catch (SQLException ex) {
            Logger.getLogger(UserDAO.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            util.DB.getInstance().putConnection(conn);
        }

        return all;
    }

    @Override
    public List<Reservation> getAllReservationsBetween(LocalDateTime from, LocalDateTime to) {
        List<Reservation> all = new ArrayList<>();
        Connection conn = util.DB.getInstance().getConnection();
        try (PreparedStatement ps = conn.prepareStatement("select * from rezervacija where (? >= datumOd and ? <= datumDo) or (? >= datumOd and ? <= datumDo)");) {
            // moze i between, ali tu ce naci preklapanje ako je kraj jednog termina pocetak drugog

            Timestamp begin = DateConverter.LocalDateTimeToTimeStamp(from);
            Timestamp end = DateConverter.LocalDateTimeToTimeStamp(to);
            ps.setTimestamp(1, begin);
            ps.setTimestamp(2, begin);
            ps.setTimestamp(3, end);
            ps.setTimestamp(4, end);

            ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                Reservation r = new Reservation();
                r.setFrom(DateConverter.TimestampToLocalDateTime(rs.getTimestamp("datumOd")));
                r.setTo(DateConverter.TimestampToLocalDateTime(rs.getTimestamp("datumDo")));
                r.setTitle(rs.getString("naziv"));
                r.parseAndSetLabs(rs.getString("prostorije"));
                r.setId(rs.getInt("id"));
                all.add(r);
            }

        } catch (SQLException ex) {
            Logger.getLogger(UserDAO.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            util.DB.getInstance().putConnection(conn);
        }

        return all;
    }

    @Override
    public boolean insertReservation(Reservation r) {
         Connection conn = util.DB.getInstance().getConnection();
        try (PreparedStatement ps = conn.prepareStatement("insert into rezervacija (naziv,datumOd,datumDo,prostorije) values(?,?,?,?)");) {
            ps.setString(1, r.getTitle());
            ps.setTimestamp(2, DateConverter.LocalDateTimeToTimeStamp(r.getFrom()));
            ps.setTimestamp(3, DateConverter.LocalDateTimeToTimeStamp(r.getTo()));
            String labsString = String.join(",", r.getLabs());
            ps.setString(4, labsString);

            return ps.executeUpdate() != 0;

        } catch (SQLException ex) {
            Logger.getLogger(UserDAO.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            util.DB.getInstance().putConnection(conn);
        }

        return false;
    }
    
}
